#include "servo.h"

#include <esp32-hal.h> // For `millis()`

ServoController::ServoController(int timer, int frequency, int pin) 
    : timer(timer), frequency(frequency), pin(pin) {
    
}

void ServoController::configure(uint32_t openDuration, int pressedValue, int releasedValue) {
    this->openDuration = openDuration;
    this->pressedValue = pressedValue;
    this->releasedValue = releasedValue;
    ESP32PWM::allocateTimer(timer);
    servo.setPeriodHertz(frequency);
    servo.attach(pin);
    releaseButton();
}

void ServoController::pressButton() {
    servo.write(pressedValue);
    buttonIsPressed = true;
    openingEndTime = millis() + openDuration;
}

void ServoController::releaseButton() {
    servo.write(releasedValue);
    buttonIsPressed = false;
}

void ServoController::loop() {
    if (buttonIsPressed && millis() > openingEndTime) {
        releaseButton();
    }
}


